package be.iminds.ilabt.jfed.experimenter_gui.util;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Resource;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.ResourceClass;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.rspec.model.*;
import be.iminds.ilabt.jfed.rspec.model.impl.BasicModelRspec;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;
import java.util.stream.Collectors;

/**
 * GuiConfigRSpecGenerator
 */
public class GuiConfigRSpecGenerator {
    @Nonnull
    public static String generateRequestRSpec(@Nonnull JFedGuiConfig guiConfig, @Nonnull Server server, int nodeCount, @Nullable String jFedResourceClassId) {
        ResourceClass chosenResourceClass;
        if (jFedResourceClassId == null) {
            List<ResourceClass> resourceClassList = guiConfig.findAllPossibleResourceClasses(server);
            chosenResourceClass = null;
            for (ResourceClass resourceClass : resourceClassList) {
                //highest weight is preferred
                if (chosenResourceClass == null || resourceClass.getWeightOrZero() > chosenResourceClass.getWeightOrZero()) {
                    chosenResourceClass = resourceClass;
                }
            }
            if (chosenResourceClass == null) {
                throw new RuntimeException("There is no resourceClass for server "+server.getId()+" that can be used.");
            }
        } else {
            chosenResourceClass = null;
            //allow just starting with the provided ID  (this is convenient because resource classes differ in suffix for different user authorities)
            for (ResourceClass resourceClass : guiConfig.getResourceClasses()) {
                if (chosenResourceClass == null && resourceClass.getId().startsWith(jFedResourceClassId))
                    chosenResourceClass = guiConfig.getResourceClass(jFedResourceClassId);
                if (resourceClass.getId().equalsIgnoreCase(jFedResourceClassId))
                    chosenResourceClass = guiConfig.getResourceClass(jFedResourceClassId);
            }
            if (chosenResourceClass == null) {
                throw new RuntimeException("No resourceClass \""+jFedResourceClassId+"\" found. All resource classes: "+
                        guiConfig.getResourceClasses().stream().map(ResourceClass::getId).collect(Collectors.joining(", ")));
            }
        }

        Resource chosenResource = guiConfig.getDefaultResource(chosenResourceClass);
        if (chosenResource == null) {
            throw new RuntimeException("No Resource found for ResourceClass \""+jFedResourceClassId+"\" and server "+server.getId()+"");
        }
        if (chosenResource.getRspecElementName() != null &&
                !chosenResource.getRspecElementName().equalsIgnoreCase("node")) {
            throw new RuntimeException("Resource for ResourceClass \""+jFedResourceClassId+"\" and " +
                    "server "+server.getId()+" found, but has unsupported RSpec element name: " +
                    "\""+chosenResource.getRspecElementName()+"\"");
        }

        RspecFactory factory = RspecFactoryFactory.getRspecFactoryInstance(ModelRspecType.BASIC);
        ModelRspec model = factory.createModelRspec("request");
        for (int i = 0; i < nodeCount; i++) {
            RspecNode node = factory.createNode(model);
            node.setComponentManagerId(server.getDefaultComponentManagerUrn());
            node.setClientId("n"+i);

            node.setSliverTypeName(chosenResource.getSliverType());

            if (chosenResource.getHardwareType() != null) {
                node.getHardwareTypes().add(new HardwareType(chosenResource.getHardwareType()));
            }
            if (chosenResource.getDiskImage() != null) {
                node.setSliverTypeDiskImage(chosenResource.getDiskImage());
            }

            model.addNode(node);
        }
        return model.toGeni3Rspec();
    }
}
